SystemOrganization addCategory: #SlotTests!

Trait named: #TOne
	uses: {}
	category: 'SlotTests'!

!TOne methodsFor: 'as yet unclassified' stamp: 'MartinDias 8/1/2012 00:06'!
one
	^ 1! !

Trait named: #TTwo
	uses: {}
	category: 'SlotTests'!

!TTwo methodsFor: 'as yet unclassified' stamp: 'MartinDias 8/1/2012 00:11'!
two
	^ 2! !

SharedPool subclass: #TestSharedPool
	instanceVariableNames: ''
	classVariableNames: 'One'
	poolDictionaries: ''
	category: 'SlotTests'!

!TestSharedPool class methodsFor: 'class initialization' stamp: 'MartinDias 8/7/2012 02:06'!
initialize
	One := 1! !

TestCase subclass: #SlotClassBuilderTest
	instanceVariableNames: 'aClass instance anotherClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SlotTests'!

SlotClassBuilderTest subclass: #SlotClassBuilderAnnouncementsTest
	instanceVariableNames: 'announcement announcementsCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SlotTests'!

!SlotClassBuilderAnnouncementsTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:01'!
assertOnlyOneAnnouncementOf: anAnnouncement within: aBlock
	SystemAnnouncer uniqueInstance weak on: anAnnouncement send: #countAnnouncements: to: self.
	aBlock value.
	self assert: announcementsCount <= 1! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:00'!
countAnnouncements: anAnnouncement 
	announcementsCount := announcementsCount + 1! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:00'!
saveAnnouncement: anAnnouncement
	announcement := anAnnouncement! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:00'!
saveAnnouncementOnce: anAnnouncement
	announcement ifNotNil: [ ^self ].
	announcement := anAnnouncement! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'running' stamp: 'MartinDias 6/3/2013 17:19'!
setUp
	super setUp.
	announcementsCount := 0.! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'tests-announcements' stamp: 'MartinDias 6/3/2013 16:55'!
testClassAddedAnnounced
	| classAdded |
	SystemAnnouncer uniqueInstance weak on: ClassAdded send: #saveAnnouncement: to: self.
	
	classAdded := self buildClassWithLayout: PointerLayout.
	
	self assert: announcement classAdded equals: classAdded.! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'tests-announcements' stamp: 'MartinDias 6/3/2013 16:55'!
testClassAddedAnnouncedOnlyOnce
	self assertOnlyOneAnnouncementOf: ClassAdded within: [
		"We create a new class without slots"
		self buildClassWithLayout: PointerLayout.
	]! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'tests-announcements' stamp: 'MartinDias 6/3/2013 16:55'!
testClassFormatChangedAnnounced
	| classAdded classModified |
	SystemAnnouncer uniqueInstance weak on: ClassModifiedClassDefinition send: #saveAnnouncementOnce: to: self.
	
	"We create a new class without slots"
	classAdded := self buildClassWithLayout: PointerLayout.
	
	"now we extend it with slots.  The class definition should've changed"
	classModified := self buildClassWithLayout: WeakLayout.
	
	self assert: announcement newClassDefinition equals: classModified.
	self assert: announcement oldClassDefinition format equals: classAdded format.
	self assert: announcement newClassDefinition format equals: classModified format.! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'tests-announcements' stamp: 'MartinDias 6/3/2013 16:55'!
testClassModifiedAnnounced
	| classAdded classModified |
	SystemAnnouncer uniqueInstance weak on: ClassModifiedClassDefinition send: #saveAnnouncementOnce: to: self.
	
	"We create a new class without slots"
	classAdded := self buildClassWithLayout: PointerLayout.
	
	"now we extend it with slots.  The class definition should've changed"
	classModified := self buildClassWithLayout: PointerLayout andSlots: { #aSlot }.
	
	self assert: announcement newClassDefinition equals: classModified.
	self assert: announcement oldClassDefinition allInstVarNames equals: classAdded allInstVarNames.! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'tests-announcements' stamp: 'MartinDias 6/3/2013 16:55'!
testClassModifiedAnnouncedOnlyOnce
	
	self assertOnlyOneAnnouncementOf: ClassModifiedClassDefinition within: [
		
		"We create a new class without slots"
		self buildClassWithLayout: PointerLayout.
		
		"now we extend it with slots.  The class definition should've changed"
		self buildClassWithLayout: PointerLayout andSlots: { #aSlot }.
	]! !

!SlotClassBuilderAnnouncementsTest methodsFor: 'tests-announcements' stamp: 'MartinDias 6/3/2013 16:55'!
testClassRecategorizedAnnounced

	SystemAnnouncer uniqueInstance weak 
		on: ClassRecategorized 
		send: #saveAnnouncementOnce: 
		to: self.
	
	aClass := self buildClassWith: [:builder|
		builder category: self aCategory ].	
	anotherClass := self buildClassWith: [:builder|
		builder category: self anotherCategory ].
	
	self assert: announcement oldCategory equals: self aCategory.
	self assert: announcement newCategory equals: self anotherCategory.
	self assert: announcement classRecategorized == anotherClass.! !

SlotClassBuilderTest subclass: #SlotClassBuilderSilentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SlotTests'!

!SlotClassBuilderSilentTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:07'!
assertClassBuiltWith: blockToBuildAClass isEqualToClassBuiltWith: blockToBuildAnotherClass
	
	aClass := self buildClassWith: [ :builder | 
		builder 
			name: self aClassName;
			in: blockToBuildAClass ].
		
	anotherClass := self buildClassWith: [ :builder | 
		builder 
			name: self anotherClassName;
			in: blockToBuildAnotherClass ].
	
	self assert: aClass layout equals: aClass layout.	
	self assert: aClass layout equals: anotherClass layout.
	self assert: aClass layout hash equals: anotherClass layout hash.! !

!SlotClassBuilderSilentTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:07'!
assertClassBuiltWith: blockToBuildAClass isNotEqualToClassBuiltWith: blockToBuildAnotherClass
		
	aClass := self buildClassWith: [ :builder | 
		builder 
			name: self aClassName;
			in: blockToBuildAClass ].
		
	anotherClass := self buildClassWith: [ :builder | 
		builder 
			name: self anotherClassName;
			in: blockToBuildAnotherClass ].
	
	self assert: aClass layout ~= anotherClass layout.
	self assert: aClass layout hash ~= anotherClass layout hash.! !

!SlotClassBuilderSilentTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:07'!
assertInvalidClassName: aName

	self should: [
		self buildClassWith: [ :builder | 
			builder name: aName ] ]
		raise: InvalidGlobalNameConflict.
	self flag: 'should remove class if not raised'.
		
	self should: [
		PharoClassInstaller 
			validateClassName: aName ]
		raise: InvalidGlobalNameConflict! !

!SlotClassBuilderSilentTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:06'!
runCase

	SystemAnnouncer uniqueInstance suspendAllWhile: [ super runCase ]
! !

!SlotClassBuilderSilentTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:07'!
should: superclassBlock extendWith: subclassBlock

	| superclass |
	superclass := self buildClassWith: superclassBlock.

	self buildClassWith: [ :builder | 
		builder 
			superclass: superclass;
			name: self anotherClassName;
			in: subclassBlock ]! !

!SlotClassBuilderSilentTest methodsFor: 'helpers' stamp: 'MartinDias 6/3/2013 17:07'!
shouldnt: superclassBlock extendWith: subclassBlock

	| superclass |
	superclass := self buildClassWith: superclassBlock.

	self should: [
		self buildClassWith: [ :builder | 
			builder 
				superclass: superclass;
				name: self anotherClassName;
				in: subclassBlock ] ]
		raise: PrimitiveTypeConflict! !

!SlotClassBuilderSilentTest methodsFor: 'tests-class slots' stamp: 'MartinDias 6/3/2013 17:07'!
testAddClassSlotAndMigrate

	| classAdded classModified |
	classAdded := 
		self buildClassWith: [ :builder | 
			builder classSlots: #(a) ].

	classAdded instVarNamed: #a put: $A.
	
	classModified := 
		self buildClassWith: [ :builder | 
			builder classSlots: #(a b) ].
	
	self assert: (classModified instVarNamed: #a) equals: $A.
	self assert: (classModified instVarNamed: #b) isNil.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-integration' stamp: 'CamilloBruni 6/28/2013 14:45'!
testAddInstVarNamed
	"Add instance variables using the builder interface"
	
	aClass := self buildClassWith: [ :builder |
		builder 
			name: self aClassName;
			slots: #( x )
		].

	self assert: aClass instVarNames = #(x).	


	anotherClass := self buildClassWith: [ :builder |
		builder 
			superclass: aClass;
			name: self anotherClassName;
			slots: #( y )
		].
	
	self assert: anotherClass instVarNames = #(y).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-integration' stamp: 'CamilloBruni 6/28/2013 14:45'!
testAddInstVarNamedClassInterface

	"Add instance variables using the class interface"
	
	aClass := self buildClassWith: [ :builder |
		builder name: self aClassName 
		].
	aClass addInstVarNamed: 'x'.

	self assert: aClass instVarNames = #(x).	


	anotherClass := self buildClassWith: [ :builder |
		builder 
			superclass: aClass;
			name: self anotherClassName.
		].
	anotherClass addInstVarNamed: 'y'.
	
	self assert: anotherClass instVarNames = #(y).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared pools' stamp: 'MartinDias 6/3/2013 17:07'!
testAddSharedPool

	self buildClassWith: [ :builder | 
		builder sharedPools: '' ].

	aClass := self buildClassWith: [ :builder | 
		builder sharedPools: 'TestSharedPool' ].
	
	self assert: (aClass sharedPools includes: TestSharedPool).
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared variables' stamp: 'MartinDias 6/3/2013 17:07'!
testAddSharedVariable

	self buildClassWith: [ :builder | 
		builder sharedVariables: '' ].

	aClass := self buildClassWith: [ :builder | 
		builder sharedVariables: 'Var' ].
	
	self assert: aClass classVarNames equals: #(Var).
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-migrations' stamp: 'MartinDias 6/3/2013 17:08'!
testAddSlotAndMigrate
	"We create a class without slots and create an instance; then we add a slot and check the instance can hold a value."
	| classAdded |
	classAdded := self buildClassWithLayout: PointerLayout.
	
	instance := classAdded new.
	
	"now we extend it with slots.  The class definition should've changed"
	self buildClassWithLayout: PointerLayout andSlots: { #aSlot }.
	
	"it should have an instance variable"
	instance instVarAt: 1 put: 125.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-migrations' stamp: 'MartinDias 6/3/2013 17:08'!
testAddSlotPropagateAndMigrate
	"We create a class without slots and a subclass, and create an instance of the latter; then we add a slot to superclass and check the instance can hold a value."

	| superclass subclass |
	superclass := self buildClassWithLayout: PointerLayout.
	subclass := self buildClassWith: [ :builder |
		builder 
			superclass: superclass;
			name: self anotherClassName.
		 ].
	
	instance := subclass new.
	
	"now we extend it with slots.  The class definition should've changed"
	superclass := self buildClassWithLayout: PointerLayout andSlots: { #aSlot }.
	
	"it should have an instance variable"
	instance instVarNamed: 'aSlot' put: 42.
	self assert: (instance instVarNamed: 'aSlot') = 42.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-environment' stamp: 'MartinDias 6/28/2013 12:33'!
testBasicEnvironment
	| environment |
	environment := SystemDictionary new.
	environment at: #SystemOrganization put: SystemOrganizer new.

	aClass := self buildClassWith: [ :builder | 
		builder environment: environment ].

	self assert: (aClass environment == environment).
	self assert: (environment includesKey: aClass name).	
	self assert: (environment at: aClass name) == aClass.
	
	self deny: (Smalltalk globals includesKey: aClass name).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-valid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testByteCanExtendByte

	self should: [ :builder | 
			builder layoutClass: ByteLayout ]
		extendWith: [ :builder | 
			builder layoutClass: ByteLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testByteCannotExtendPointerWithFields

	self shouldnt: [ :builder | 
			builder 
				layoutClass: PointerLayout;
				slots: { #a }. ]
		extendWith: [ :builder | 
			builder layoutClass: ByteLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testByteCannotExtendWord

	self shouldnt: [ :builder | 
			builder layoutClass: WordLayout ]
		extendWith: [ :builder | 
			builder layoutClass: ByteLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 17:07'!
testCannotBeRecompiled

	| superclass |
	superclass := self buildClassWith: [ :builder | 
		builder 
			superclass: Object;
			name: self aClassName ].

	self buildClassWith: [ :builder | 
		builder 
			superclass: superclass;
			name: self anotherClassName ].

	self 
		should: [ 
			self buildClassWith: [ :builder | 
				builder 
					layoutClass: VariableLayout;
					name: self aClassName ] ]
		raise: Error
		description: 'Old class builder raises: X cannot be recompiled'
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-migrations' stamp: 'MartinDias 6/3/2013 17:12'!
testChangingFormatKeepsMethod

	aClass := self buildClassWith: [ :builder | 
		builder 
			name: self aClassName;
			layoutClass: PointerLayout ].

	aClass compile: 'a ^self'.
	self assert: (aClass methodDictionary includesKey: #a).
	
	anotherClass := self buildClassWith: [ :builder | 
		builder 
			name: self aClassName;
			layoutClass: VariableLayout .
			].

	self assert: (anotherClass methodDictionary includesKey: #a).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 16:59'!
testCheckAllLayouts
	self shouldnt: [SlotVirus checkAllLayouts] raise: Error! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 16:58'!
testClassNameMustBeCapitalized

	self assertInvalidClassName: #aNotCapitalizedClassName! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 16:58'!
testClassNameMustBeSymbol

	self assertInvalidClassName: 'NotSymbolClassName'! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 16:58'!
testClassNameWithInvalidCharacter

	self assertInvalidClassName: #'Invalid-ClassName'! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 17:07'!
testClassSlotDuplicationConflict

	aClass := self buildClassWith: [ :builder | 
		builder classSlots: { #a } ].

	self should: [ 
		self buildClassWith: [ :builder | 
			builder 
				superclass: aClass;
				name: self anotherClassName;
				classSlots: { #a } ] ]
		raise: SlotDuplicationConflict ! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 17:07'!
testClassSlotWithReservedName

	#(#self #super #thisContext #true #false #nil) do: [:reservedName |
		self should: [ 
			self buildClassWith: [ :builder | 
				builder classSlots: { reservedName } ] ]
			raise: InvalidSlotNameConflict ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/3/2013 17:07'!
testClassWithClassTrait

	aClass := self buildClassWith: [ :builder | 
		builder classTraitComposition: TOne ].
	
	self assert: (aClass class canUnderstand: #one).
	self assert: aClass one equals: 1.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/3/2013 17:07'!
testClassWithTrait

	aClass := self buildClassWith: [ :builder | 
		builder traitComposition: TOne ].
	
	self assert: (aClass canUnderstand: #one).
	self assert: aClass new one equals: 1.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 16:59'!
testCompiledMethodLayout
	self assert: (CompiledMethod layout isKindOf: CompiledMethodLayout).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 17:07'!
testDirectCircularHierarchyError
	"Tests an error is raised when trying to create a heirarchy A<-A"
	
	| classA classB |
	classA := self buildClassWith: [ :builder |
			builder 
				superclass: Object;
				name: self aClassName ].

	self should: [ 
		self buildClassWith: [ :builder | 
			builder 
				superclass: classA;
				name: self aClassName ] ]
		raise: CircularHierarchyConflict ! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 17:07'!
testIndirectCircularHierarchyError
	"Tests an error is raised when trying to create a heirarchy A<-B<-A"
	
	| classA classB |
	classA := self buildClassWith: [ :builder |
			builder 
				superclass: Object;
				name: self aClassName ].

	classB := self buildClassWith: [ :builder |
			builder 
				superclass: classA;
				name: self anotherClassName ].

	self should: [ 
		self buildClassWith: [ :builder | 
			builder 
				superclass: classB;
				name: self aClassName ] ]
		raise: CircularHierarchyConflict ! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layout equality' stamp: 'MartinDias 6/3/2013 16:59'!
testLayoutEquals

	self layoutClasses do: [:aLayoutClass |
		self 
			assertClassBuiltWith: [:builder | builder layoutClass: aLayoutClass ] 
			isEqualToClassBuiltWith: [:builder | builder layoutClass: aLayoutClass ] ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layout equality' stamp: 'MartinDias 6/3/2013 16:59'!
testLayoutNotEquals
 
	self layoutClasses do: [:aLayoutClass |
		(self layoutClasses copyWithout: aLayoutClass) do: [:anotherLayoutClass |
			self 
				assertClassBuiltWith: [:builder | builder layoutClass: aLayoutClass ] 
				isNotEqualToClassBuiltWith: [:builder | builder layoutClass: anotherLayoutClass ]]]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layout equality' stamp: 'MartinDias 6/3/2013 16:59'!
testLayoutWithSlotsEquals

	self layoutClassesWithSlots do: [:aLayoutClass |
		self 
			assertClassBuiltWith: [:builder | 
				builder 
					layoutClass: aLayoutClass;
					slots: { #field1}  ]
			isEqualToClassBuiltWith: [:builder | 
				builder 
					layoutClass: aLayoutClass;
					slots: { #field1}  ].]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layout equality' stamp: 'MartinDias 6/3/2013 16:59'!
testLayoutWithSlotsNotEquals

	self layoutClassesWithSlots do: [:aLayoutClass |
		self 
			assertClassBuiltWith: [:builder | 
				builder 
					layoutClass: aLayoutClass;
					slots: { #field1}  ]
			isNotEqualToClassBuiltWith: [:builder | 
				builder 
					layoutClass: aLayoutClass;
					slots: { #field2}  ]]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/3/2013 17:07'!
testModifyClassTraitComposition
	"Tests that when modifing the trait composition on class-side, the old methods are removed from the method dictionary, and the new ones are added."

	self buildClassWith: [ :builder | 
		builder classTraitComposition: TOne ].

	aClass := self buildClassWith: [ :builder | 
		builder classTraitComposition: TTwo ].
	
	self deny: (aClass class canUnderstand: #one).
	self assert: (aClass class canUnderstand: #two).
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/3/2013 17:07'!
testModifyTraitComposition
	"Tests that when modifing the trait composition, the old methods are removed from the method dictionary, and the new ones are added."

	self buildClassWith: [ :builder | 
		builder traitComposition: TOne ].

	aClass := self buildClassWith: [ :builder | 
		builder traitComposition: TTwo ].
	
	self deny: (aClass canUnderstand: #one).
	self assert: (aClass canUnderstand: #two).
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 17:08'!
testNewPointerClass

	aClass := self buildClassWithLayout: PointerLayout.
	
	self deny: aClass isCompact.
	self assert: aClass isPointers.
	self assert: aClass isFixed.
	self assert: aClass instVarNames isEmpty.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 17:08'!
testNewPointerClassWithVariables

	aClass := self 
		buildClassWithLayout: PointerLayout
		andSlots: { #id. #name }.
	
	self assert: (aClass instVarNames includes: 'id').
	self assert: (aClass instVarNames includes: 'name').! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 17:08'!
testNewVariableByteClass

	aClass := self buildClassWithLayout: ByteLayout.
	
	self deny: aClass isCompact.
	self deny: aClass isPointers.
	self deny: aClass isFixed.
	self assert: aClass isBytes.
	self assert: aClass isVariable.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 17:08'!
testNewVariableWordClass

	aClass := self buildClassWithLayout: WordLayout.
	
	self deny: aClass isCompact.
	self deny: aClass isPointers.
	self deny: aClass isFixed.
	self assert: aClass isWords.
	self assert: aClass isVariable.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 17:08'!
testNewWeakClass

	aClass := self buildClassWithLayout: WeakLayout.
	
	self deny: aClass isCompact.
	self assert: aClass isPointers.
	self deny: aClass isFixed.
	self assert: aClass isWords.
	self assert: aClass isWeak.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-valid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testPointerCanExtendPointer

	self should: [ :builder | 
			builder layoutClass: PointerLayout ]
		extendWith: [ :builder | 
			builder layoutClass: PointerLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-valid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testPointerCanExtendVariable

	self should: [ :builder | 
			builder layoutClass: VariableLayout ]
		extendWith: [ :builder | 
			builder layoutClass: PointerLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testPointerCannotExtendByte

	self shouldnt: [ :builder | 
			builder layoutClass: ByteLayout ]
		extendWith: [ :builder | 
			builder layoutClass: PointerLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testPointerCannotExtendWord

	self shouldnt: [ :builder | 
			builder layoutClass: WordLayout ]
		extendWith: [ :builder | 
			builder layoutClass: PointerLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-valid extensions' stamp: 'MartinDias 6/3/2013 17:07'!
testPointerWithVariableExtensionIsNowVariable
	"This test case reproduces a real bug."

	| superclass subclass |
	superclass := self buildClassWith: [ :builder | 
		builder 
			superclass: Object;
			name: self aClassName ].

	subclass := self buildClassWith: [ :builder | 
		builder 
			superclass: superclass;
			layoutClass: VariableLayout;
			name: self anotherClassName ].

	superclass := self buildClassWith: [ :builder | 
		builder layoutClass: VariableLayout ].

	self assert: superclass isVariable.
	self assert: subclass isVariable.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-migrations' stamp: 'MartinDias 6/3/2013 17:08'!
testRedefineSuperclass
	"This case reproduces a MNU found loading Moose."

	| superclass |
	superclass := self buildClassWithLayout: PointerLayout.
	self buildClassWith: [ :builder |
		builder 
			superclass: superclass;
			layoutClass: ByteLayout;
			name: self anotherClassName.
		 ].

	self buildClassWithLayout: PointerLayout.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-class slots' stamp: 'MartinDias 6/3/2013 17:07'!
testRemoveClassSlotAndMigrate

	| classAdded classModified |
	classAdded := 
		self buildClassWith: [ :builder | 
			builder classSlots: #(a b) ].

	classAdded instVarNamed: #a put: $A.
	
	classModified := 
		self buildClassWith: [ :builder | 
			builder classSlots: #(a) ].
	
	self assert: (classModified instVarNamed: #a) equals: $A.
	self should: [ classModified instVarNamed: #b ] raise: Error
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/3/2013 17:07'!
testRemoveClassTrait
	"Tests that when removing a trait from class-side, its methods are removed from the method dictionary."

	self buildClassWith: [ :builder | 
		builder classTraitComposition: TOne ].

	aClass := self buildClassWith: [ :builder | 
		builder classTraitComposition: {} ].
	
	self deny: (aClass class canUnderstand: #one).
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-integration' stamp: 'CamilloBruni 6/28/2013 14:46'!
testRemoveInstVarNamed

	aClass := self buildClassWith: [ :builder |
		builder 
			name: self aClassName;
			slots: #(x y z) 
		].
	aClass  := self buildClassWith: [ :builder |
		builder 
			name: self aClassName;
			slots: #(x   z) 
		].

	self assert: aClass instVarNames = #(x z).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-integration' stamp: 'CamilloBruni 6/28/2013 14:46'!
testRemoveInstVarNamedClassInterface

	aClass := self buildClassWith: [ :builder |
		builder 
			name: self aClassName;
			slots: #(x y z) 
		].
	aClass removeInstVarNamed: #y.

	self assert: aClass instVarNames = #(x z).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared pools' stamp: 'MartinDias 6/3/2013 17:07'!
testRemoveSharedPool

	self buildClassWith: [ :builder | 
		builder sharedPools: 'TestSharedPool' ].

	aClass := self buildClassWith: [ :builder | 
		builder sharedPools: '' ].
	
	self assert: aClass sharedPools isEmpty
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared variables' stamp: 'MartinDias 6/3/2013 17:07'!
testRemoveSharedVariable

	self buildClassWith: [ :builder | 
		builder sharedVariables: 'Var' ].

	aClass := self buildClassWith: [ :builder | 
		builder sharedVariables: '' ].
	
	self assert: aClass classVarNames isEmpty! !

!SlotClassBuilderSilentTest methodsFor: 'tests-migrations' stamp: 'MartinDias 6/3/2013 17:08'!
testRemoveSlotAndMigrate
	"We create a class with a slot and create an instance; then we remove the slot and check the instance doesn't hold the value."
	| classAdded |
	classAdded := self buildClassWithLayout: PointerLayout andSlots: { #aSlot }.

	instance := classAdded new.
	"it should have an instance variable"
	instance instVarAt: 1 put: 125.
	
	"now we extend it with slots.  The class definition should've changed"
	self buildClassWithLayout: PointerLayout.
	
	self should: [ instance instVarAt: 1 ] raise: Error.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/28/2013 16:11'!
testRemoveTrait
	"Tests that when removing a trait from a class, its methods are removed from the method dictionary."
	
	self assert: TOne users isEmpty.
	
	aClass := self buildClassWith: [ :builder | 
		builder traitComposition: TOne ].
	
	self assert: TOne users asArray equals: { aClass }.
	
	aClass := self buildClassWith: [ :builder | 
		builder traitComposition: {} ].
	
	self assert: TOne users isEmpty.
	self deny: (aClass canUnderstand: #one).
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-migrations' stamp: 'MartinDias 6/3/2013 17:13'!
testShiftSlotAndMigrate

	aClass := self buildClassWithLayout: PointerLayout andSlots: { #a. #c }.
	aClass compile: 'a ^a'.
	aClass compile: 'c ^c'.
	
	instance := aClass new.
	instance instVarNamed: #a put: 1.
	instance instVarNamed: #c put: 3.

	"now we extend it with slots.  The class definition should've changed"
	self buildClassWithLayout: PointerLayout andSlots: { #a. #b. #c }.

	"it should have added an instance variable with nil in the middle"
	self assert: (instance instVarAt: 1) equals: 1.
	self assert: (instance instVarAt: 2) equals: nil.
	self assert: (instance instVarAt: 3) equals: 3.
	
	self assert: (instance instVarNamed: #a) equals: 1.
	self assert: (instance instVarNamed: #b) equals: nil.
	self assert: (instance instVarNamed: #c) equals: 3.
	
	"it should have migrated methods"
	self assert: instance a equals: 1.
	self assert: instance c equals: 3.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 17:08'!
testSlotBuilderBuildsClass

	aClass := self buildClassWithLayout: PointerLayout.
	
	self assert: (Smalltalk classNamed: self aClassName) == aClass.
	self assert: aClass name = self aClassName.
	self deny: aClass layout isNil.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 17:07'!
testSlotDuplicationConflict

	aClass := self buildClassWith: [ :builder | 
		builder slots: { #a } ].

	self should: [ 
		self buildClassWith: [ :builder | 
			builder 
				superclass: aClass;
				name: self anotherClassName;
				slots: { #a } ] ]
		raise: SlotDuplicationConflict ! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 17:07'!
testSlotWithReservedName

	#(#self #super #thisContext #true #false #nil) do: [:reservedName |
		self should: [ 
			self buildClassWith: [ :builder | 
				builder slots: { reservedName } ] ]
			raise: InvalidSlotNameConflict ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-layouts' stamp: 'MartinDias 6/3/2013 16:59'!
testSmallIntegerLayout
	self assert: (SmallInteger layout isKindOf: SmallIntegerLayout).! !

!SlotClassBuilderSilentTest methodsFor: 'tests-class slots' stamp: 'MartinDias 6/3/2013 17:14'!
testSwitchClassSlotAndMigrate

	| classAdded classModified |	
	classAdded := 
		self buildClassWith: [ :builder | 
			builder classSlots: #(a b) ].

	"Add methods"	
	classAdded class 
		compile: 'a ^a';
		compile: 'b ^b'.

	"Put values"
	classAdded instVarNamed: #a put: $A.
	classAdded instVarNamed: #b put: $B.

	"Modify the class"	
	classModified := 
		self buildClassWith: [ :builder | 
			builder classSlots: #(b a) ].
	
	"The instance was migrated"
	self assert: (classModified instVarNamed: #a) equals: $A.
	self assert: (classModified instVarNamed: #b) equals: $B.
	
	"The methods were migrated"
	self assert: classModified a equals: $A.
	self assert: classModified b equals: $B.
! !

!SlotClassBuilderSilentTest methodsFor: 'tests-migrations' stamp: 'MartinDias 6/3/2013 17:15'!
testSwitchSlotsAndMigrate

	aClass := self buildClassWithLayout: PointerLayout andSlots: { #a. #b }.
	aClass 
		compile: 'a ^a';
		compile: 'b ^b'.

	instance := aClass new.
	instance instVarNamed: #a put: $A.
	instance instVarNamed: #b put: $B.

	"now we switch the slots. The class definition should've changed"
	self buildClassWithLayout: PointerLayout andSlots: { #b. #a }.

	self assert: (instance instVarAt: 1) equals: $B.
	self assert: (instance instVarAt: 2) equals: $A.
	
	self assert: (instance instVarNamed: #a) equals: $A.
	self assert: (instance instVarNamed: #b) equals: $B.

	"it should have migrated methods"
	self assert: instance a equals: $A.
	self assert: instance b equals: $B.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/28/2013 18:03'!
testTraitUsersAfterClassReshape

	aClass := self buildClassWith: [ :builder | 
		builder
			name: self aClassName ].

	anotherClass := self buildClassWith: [ :builder | 
		builder
			name: self anotherClassName;
			superclass: aClass;
			traitComposition: TOne;
			classTraitComposition: TTwo ].
	
	self assert: TOne users asArray equals: { anotherClass }.
	self assert: TTwo users asArray equals: { anotherClass class }.
	self deny: (aClass canUnderstand: #one).
	self assert: (anotherClass canUnderstand: #one).
	self deny: (aClass class canUnderstand: #two).
	self assert: (anotherClass class canUnderstand: #two).
	
	
	"reshape the super class"
	aClass := self buildClassWith: [ :builder | 
		builder
			name: self aClassName;
			slots: #( x y z ) ].
		
	self assert: TOne users asArray equals: { anotherClass }.
	self assert: TTwo users asArray equals: { anotherClass class }.
	self deny: (aClass canUnderstand: #one).
	self assert: (anotherClass canUnderstand: #one).
	self deny: (aClass class canUnderstand: #two).
	self assert: (anotherClass class canUnderstand: #two).

! !

!SlotClassBuilderSilentTest methodsFor: 'tests-traits' stamp: 'MartinDias 6/28/2013 18:08'!
testTraitUsersAfterMetaclassReshape

	aClass := self buildClassWith: [ :builder | 
		builder
			name: self aClassName ].

	anotherClass := self buildClassWith: [ :builder | 
		builder
			name: self anotherClassName;
			superclass: aClass;
			traitComposition: TOne;
			classTraitComposition: TTwo ].
	
	self assert: TOne users asArray equals: { anotherClass }.
	self assert: TTwo users asArray equals: { anotherClass class }.
	self deny: (aClass canUnderstand: #one).
	self assert: (anotherClass canUnderstand: #one).
	self deny: (aClass class canUnderstand: #two).
	self assert: (anotherClass class canUnderstand: #two).
	
	"reshape the super class"
	aClass := self buildClassWith: [ :builder | 
		builder
			name: self aClassName;
			classSlots: #( x y z ) ].
		
	self assert: TOne users asArray equals: { anotherClass }.
	self assert: TTwo users asArray equals: { anotherClass class }.
	self deny: (aClass canUnderstand: #one).
	self assert: (anotherClass canUnderstand: #one).
	self deny: (aClass class canUnderstand: #two).
	self assert: (anotherClass class canUnderstand: #two).

! !

!SlotClassBuilderSilentTest methodsFor: 'tests-errors' stamp: 'MartinDias 6/3/2013 16:58'!
testValidateClassName		
		
	PharoClassInstaller 
		validateClassName: #GoodClassNameThatShouldNotExist! !

!SlotClassBuilderSilentTest methodsFor: 'tests-valid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testVariableCanExtendPointer

	self should: [ :builder | 
			builder layoutClass: PointerLayout ]
		extendWith: [ :builder | 
			builder layoutClass: VariableLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-valid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testVariableCanExtendVariable

	self should: [ :builder | 
			builder layoutClass: VariableLayout ]
		extendWith: [ :builder | 
			builder layoutClass: VariableLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testVariableCannotExtendByte

	self shouldnt: [ :builder | 
			builder layoutClass: ByteLayout ]
		extendWith: [ :builder | 
			builder layoutClass: VariableLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testVariableCannotExtendWord

	self shouldnt: [ :builder | 
			builder layoutClass: WordLayout ]
		extendWith: [ :builder | 
			builder layoutClass: VariableLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-class slots' stamp: 'MartinDias 6/3/2013 17:07'!
testWithClassSlots

	aClass := self buildClassWith: [ :builder | 
		builder classSlots: #(a) ].
	
	self assert: aClass class instVarNames equals: #(a).	
	aClass instVarNamed: #a put: $A.
	self assert: (aClass instVarNamed: #a) equals: $A.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared pools' stamp: 'MartinDias 6/3/2013 17:15'!
testWithSharedPool

	aClass := self buildClassWith: [ :builder | 
		builder sharedPools: 'TestSharedPool' ].
	aClass class compile: 'one ^One'.
	
	self assert: (aClass sharedPools includes: TestSharedPool).
	self assert: aClass one = 1.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared variables' stamp: 'MartinDias 6/3/2013 17:15'!
testWithSharedVariable

	aClass := self buildClassWith: [ :builder | 
		builder sharedVariables: 'Var' ].
	aClass class 
		compile: 'var:x  Var:=x';
		compile: 'var  ^Var'.
	
	aClass var: true.
	
	self assert: aClass classVarNames equals: #(Var).
	self assert: aClass var.! !

!SlotClassBuilderSilentTest methodsFor: 'tests-class slots' stamp: 'MartinDias 6/3/2013 17:07'!
testWithoutClassSlots

	aClass := self buildClassWith: [ :builder | 
		builder classSlots: #() ].
	
	self assert: aClass classVarNames isEmpty! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared pools' stamp: 'MartinDias 6/3/2013 17:07'!
testWithoutSharedPools

	aClass := self buildClassWith: [ :builder | 
		builder sharedPools: '' ].
	
	self assert: aClass sharedPools isEmpty! !

!SlotClassBuilderSilentTest methodsFor: 'tests-shared variables' stamp: 'MartinDias 6/3/2013 17:07'!
testWithoutSharedVariables

	aClass := self buildClassWith: [ :builder | 
		builder sharedVariables: '' ].
	
	self assert: aClass classVarNames isEmpty! !

!SlotClassBuilderSilentTest methodsFor: 'tests-valid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testWordCanExtendWord

	self should: [ :builder | 
			builder layoutClass: WordLayout ]
		extendWith: [ :builder | 
			builder layoutClass: WordLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testWordCannotExtendByte

	self shouldnt: [ :builder | 
			builder layoutClass: ByteLayout ]
		extendWith: [ :builder | 
			builder layoutClass: WordLayout ]! !

!SlotClassBuilderSilentTest methodsFor: 'tests-invalid extensions' stamp: 'MartinDias 6/3/2013 16:59'!
testWordCannotExtendPointerWithFields

	self shouldnt: [ :builder | 
			builder 
				layoutClass: PointerLayout;
				slots: { #a } ]
		extendWith: [ :builder | 
			builder layoutClass: WordLayout ]! !

!SlotClassBuilderTest methodsFor: 'running' stamp: 'MartinDias 5/23/2013 13:19'!
aCategory
	^ 'SlotTestTmp'! !

!SlotClassBuilderTest methodsFor: 'helpers-test class names' stamp: 'MartinDias 7/31/2012 01:45'!
aClassName
	^ #NewSlotTestObject! !

!SlotClassBuilderTest methodsFor: 'running' stamp: 'MartinDias 5/23/2013 13:19'!
anotherCategory
	^ 'SlotTestTmp-Another'! !

!SlotClassBuilderTest methodsFor: 'helpers-test class names' stamp: 'MartinDias 7/31/2012 01:48'!
anotherClassName
	^ #NewSlotTestObject2! !

!SlotClassBuilderTest methodsFor: 'helpers-test class names' stamp: 'MartinDias 5/23/2013 13:19'!
buildClassWith: anUnaryBlock
	"I build a class for testing, providing basic default values, but eventually customized by the received unary block."

	^PharoClassInstaller make: [:builder|
		builder
			superclass: Object;
			name: self aClassName;
			layoutClass: PointerLayout;
			category: self aCategory.
		anUnaryBlock value: builder
		].! !

!SlotClassBuilderTest methodsFor: 'helpers-test class names' stamp: 'GuillermoPolito 7/30/2012 22:11'!
buildClassWithLayout: aClassLayout
	^self buildClassWithLayout: aClassLayout andSlots: {}! !

!SlotClassBuilderTest methodsFor: 'helpers-test class names' stamp: 'MartinDias 7/31/2012 01:39'!
buildClassWithLayout: aClassLayout andSlots: someSlots
	"
	I create a class using the Slot class builder given a class layout and a collection of slots.
	"

	^self buildClassWith: [:builder|
		builder
			layoutClass: aClassLayout;
			slots: someSlots
		].! !

!SlotClassBuilderTest methodsFor: 'running' stamp: 'MartinDias 3/28/2013 12:03'!
expectedFailures 

	^ #( testCannotBeRecompiled )! !

!SlotClassBuilderTest methodsFor: 'helpers-test class names' stamp: 'MartinDias 4/12/2013 16:08'!
layoutClasses
	^ { ByteLayout. WordLayout. PointerLayout. VariableLayout. WeakLayout }! !

!SlotClassBuilderTest methodsFor: 'helpers-test class names' stamp: 'MartinDias 4/12/2013 16:09'!
layoutClassesWithSlots
	^ { PointerLayout. VariableLayout. WeakLayout }! !

!SlotClassBuilderTest methodsFor: 'running' stamp: 'MartinDias 6/3/2013 17:19'!
tearDown
	"We remove the classes that could have been created during test run"
	super tearDown.

	SystemAnnouncer uniqueInstance suspendAllWhile: [
		{ self aClassName. self anotherClassName } do: [ :each | 
			Smalltalk globals 
				at: each 
				ifPresent: [ :class | class removeFromSystem ]]].

	SystemAnnouncer uniqueInstance unsubscribe: self.! !
TestSharedPool initialize!
